/*
 * The Cryptonit security software suite is developped by IDEALX
 * Cryptonit Team (http://IDEALX.org/ and http://cryptonit.org).
 *
 * Copyright ? 2003 IDEALX
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */


#include "Utils.hh"

#include "AddressBook.hh"


/**
 * AddressBook 
 */



namespace Cryptonit
{

    AddressBook::AddressBook()
    {
	addressBook = new DirectoryService();
    } 
    
    
    AddressBook::AddressBook( const std::string uri )
    {
	addressBook = addressBook->factory( uri );

	std::string params[] = {"", ""};
	std::string filename = "";

	if( ! addressBook->read( params ) ) {
	    std::cerr << "Cannot load address book '"<< uri <<"'."<<std::endl;
	    std::cerr << "Retry with default filename" << std::endl;

	    /* Retry with the default filename */
	    filename = appendDir( uri, ADDRESSBOOK_DEFAULT_FILENAME );

	    if( ! addressBook->read( params ) ) {
		std::cerr << "Cannot load address book '"<< uri <<"'."<<std::endl;
		delete addressBook;
		addressBook = NULL;
		return;
	    }
	}

	std::cerr << "Address book loaded from '" << (filename == "" ? uri : filename) <<"'." << std::endl;
	std::cerr << addressBook->getNbEntry() << " contact(s) added." << std::endl;
    } 


    AddressBook::~AddressBook()
    {
	if( addressBook )
	    delete addressBook;
    }

    
    bool AddressBook::save( const std::string fileName )
    {
	std::string params[] = { fileName, ""};
	return addressBook->commit( params );
    }


    bool AddressBook::addContact( Contact& contact )
    {
	Entry* infos = new Entry( contact.getInfo() );
	return addressBook->append( contact.getName(), infos );
    }


    bool AddressBook::addContact( Contact* contact )
    {
	Entry* infos = new Entry( contact->getInfo() );
	return addressBook->append( contact->getName(), infos );
    }


    bool AddressBook::removeContact( const std::string name )
    {
	return addressBook->erase( name );
    }


    Contact* AddressBook::getContact( const std::string name )
    {
	return NULL;
    }


    Entry* AddressBook::getContactDSE( const std::string name )
    {
	return addressBook->getEntry( name );
    }


    std::vector<std::string>* AddressBook::getAllContactName()
    {
	std::vector<std::string>* result = new std::vector<std::string>();

	if( addressBook != NULL ) {
	    DirectoryService::iterator itr = addressBook->begin();

	    while( itr != addressBook->end() ) {
		result->push_back( itr.first() );
		itr++;
	    }
	    return result;
	}
	else return NULL;
    }


    std::string AddressBook::getContactInfo( const std::string name, const std::string infoName )
    {
	if( addressBook != NULL ) {
	    if( name != "" && ContactInfo::isInfoSupported( infoName ) ) {

		Entry* entry = addressBook->getEntry( name );

		if( entry != NULL )
		    return( entry->getAttributeFirstValue( infoName ) );
	    }
	}
	return( std::string("") );
    }
    

    std::vector<std::string> AddressBook::getContactInfos( const std::string name, const std::string infoName )
    {
	if( addressBook != NULL ) {
	    if( name != "" && ContactInfo::isInfoSupported( infoName ) ) {

		Entry* entry = addressBook->getEntry( name );

		if( entry != NULL )
		    return( entry->getAttributeValues( name ) );
	    }
	}
	return std::vector<std::string>(0);
    }

}
