/*
 * The Cryptonit security software suite is developped by IDEALX
 * Cryptonit Team (http://IDEALX.org/ and http://cryptonit.org).
 *
 * Copyright ? 2003 IDEALX
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */


#include "Attribute.hh"

#include <string>

namespace Cryptonit {

Attribute::Attribute( )
{
    values.clear();
    type = AttributeString;
}

Attribute::Attribute( const Attribute& src )
{
    values.clear();
    values = src.values;
    type = src.type;
}

Attribute::Attribute( Attribute* attr )
{
    values.clear();
    type = attr->getType();
    values = attr->values;
}

Attribute::Attribute( AttributeType t )
{
    values.clear();
    type = t;
}


Attribute::Attribute( const std::string &val, AttributeType t)
{
    values.clear();
    type = t;
    append( val );
}

Attribute::Attribute( const std::string &val )
{
    values.clear();
    type = AttributeString;
    append( val );
}


Attribute::Attribute( const char* val, unsigned long len, AttributeType t )
{
    values.clear();
    type = t;
    append( std::string( val, len ) );
}


Attribute::Attribute( const std::vector<std::string>& val )
{
    values.clear();
    type = AttributeString;
    values = val;
}

Attribute::Attribute( const std::vector<std::string>& val, AttributeType t )
{
    values.clear();
    type = t;
    values = val;
}


Attribute::~Attribute( )
{
    values.clear();
}


AttributeType Attribute::getType() const
{
    return type;
}



bool Attribute::append( const std::string &val )
{
    if( val != "" ) {
	values.push_back( val );
	return true;
    }
    else return false;
}


bool Attribute::append( const std::string &val, AttributeType t )
{
    if( val != "" ) {
	if( t != getType() ) 
	    return false;
	else {
	    values.push_back( val );
	    return true;
	}
    }
    else return false;
}


bool Attribute::append( const char* val, unsigned long len, AttributeType t )
{
    if( val != "" ) {
	if( t != getType() ) 
	    return false;
	else {
	    values.push_back( std::string(val, len) );
	    return true;
	}
    }
    else return false;
}


bool Attribute::erase( const std::string &val )
{
    std::vector<std::string>::iterator i;

    i = values.begin();

    while( i != values.end() && *i != val ) i++;
    if( i != values.end() ) {
	values.erase( i );
	return true;
    }
    else return false;
}

bool Attribute::clear()
{
    values.clear();
    return true;
}

std::vector<std::string> Attribute::getValues()
{
    return values;
}


const std::string& Attribute::getFirstValue() const
{
    return values.front();
}



const std::string& Attribute::getLastValue() const
{
    return values.back();
}


unsigned int Attribute::getNbValues() const
{
    return values.size();
}

	
} // Namespace
