/*
 * The Cryptonit security software suite is developped by IDEALX
 * Cryptonit Team (http://IDEALX.org/ and http://cryptonit.org).
 *
 * Copyright ? 2003 IDEALX
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */


#ifndef _LDAP_HH_
#define _LDAP_HH_

#include "DirectoryService.hh"

#include <ldap.h>

#ifndef LDAP_DEFAULT_PORT
#define LDAP_DEFAULT_PORT 389
#endif

namespace Cryptonit
{

   /** 
    * Class dealing with the distant LDAP containing certificates
    * 
    * @param host 
    * @param port 
    * 
    * @return 
    */
   class ldap: public DirectoryService
   {
      private:

	 LDAP* ldap_session;

	 // Timeout in seconds for asynchronous search
	 unsigned int timeout;


	 std::string current_host;
	 unsigned int current_port;
	 std::string current_base;

	 // Scope type for search
	 int current_scope;

	 LDAP* init_ldap_session( const std::string host, int port );
	 bool add_entry( LDAP* ld, LDAPMessage *entry );

      public:
	 ldap();
	 ldap( const std::string uri );

	 /** 
	  * Connection to the ldap
	  * 
	  * @param params to be sent to the ldap
	  * 
	  * @return true if connection established
	  */
	 bool bind( const std::string params[] );

	 /** 
	  * throw a research on the ldap amount of data
	  * 
	  * @param exp the expression searched
	  * 
	  * @return true if expression found
	  */
	 bool findExpression( const std::string exp );

	 /** 
	  * getting information from the ldap
	  * 
	  * @param name the entry searched
	  * 
	  * @return data returned
	  */
	 Entry* getEntry( const std::string name );


	 unsigned int getTimeout();
	 void setTimeout( unsigned int t );


	 /** 
	  * getting the way of research
	  * 
	  * 
	  * @return 
	  */
	 int getScope();

	 /** 
	  * selection of the technic of research
	  * 
	  * @param s const representing the new scope
	  *            - LDAP_SCOPE_BASE
	  *            - LDAP_SCOPE_BASE
	  *            - LDAP_SCOPE_ONELEVEL
	  *            - LDAP_SCOPE_ONELEVEL
	  *            - LDAP_SCOPE_SUBTREE
	  *
	  */
	 void setScope( int s );
	 void setScope( const std::string s );
   };

} // Namespace

#endif
