/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you want to add, delete, or rename functions or slots, use
** Qt Designer to update this file, preserving your code.
**
** You should not define a constructor or destructor in this file.
** Instead, write your code in functions called init() and destroy().
** These will automatically be called by the form's constructor and
** destructor.
*****************************************************************************/
#include "Pkcs12Dlg.h"
#include "User.hh"
#include "Certificate.hh"
#include "pkcs12.hh"
#include "pkcs10.hh"
#include "pkcs8.hh"
#include "Crl.hh"
#include "env.hh"

#include <qfiledialog.h>
#include <qinputdialog.h>
#include <qmessagebox.h>

#include "PasswordDlg.h"
#include "AddCrlDlg.h"
#include "TokenKeySelectionDlg.h"


Cryptonit::Certificate AddIdentity::getCertificate()
{
  return cert;
}


Cryptonit::pkcs12 AddIdentity::getKey()
{
  return p12;
}


QString AddIdentity::getIdentity()
{
  return identity;
}


void AddIdentity::init( Cryptonit::User * u)
{
  user = u;
}


void AddIdentity::onOk()
{
  /* Add from a PKCS12 */
  if( p12Radio->isChecked())
  {

    Pkcs12Dlg p12dlg(this);
    p12dlg.init(true);

    if( p12dlg.exec() == QDialog::Accepted)
    {

      // If there is no identity name, set if to 'default'
      if( p12dlg.getIdentity() == "" )
        identity = "Default";
      else
        identity = p12dlg.getIdentity();

      int res;
      QString errorMsg;
      //QString pkcs12FileName = user->getCertificatesDir() + QDir::separator() +
      QString pkcs12FileName = p12dlg.getPkcs12Filename();
      
      res = user->addIdentityFromP12( identity.latin1(),
                                      pkcs12FileName.latin1(),
                                      p12dlg.getPkcs12Password().latin1() );

      switch( res )
      {
      case -1: errorMsg = "No foi possvel carregar o PKCS#12.";
        break;
      case -2: errorMsg = "Esse arquivo PKCS#12 j existe para esse usurio.";
        break;
      case -3: errorMsg = "Essa identidade j existe na lista, por favor, utilize um nome diferente.";
        break;
      case -4: errorMsg = "Ocorreu um erro ao gravar o certificado no disco.";
        break;
      case -5: errorMsg = "No foi possvel adicionar as informaes de identidade na lista de contatos.";
        break;
      case -6: errorMsg = "No foi possvel gravar no disco o certitificado CA includo no PKCS#12.";
        break;
      case -7: errorMsg = "No foi possvel copiar o PKCS#12 no seu diretrio de usurio.";
        break;
      }

      if( !errorMsg.isEmpty())
      {
        QMessageBox::critical(this, "Erro", errorMsg);
        accept();
        return;
      }

      p12.load( p12dlg.getPkcs12Filename().latin1(), p12dlg.getPkcs12Password().latin1() );
      cert = p12.getCertificate();
      //downloading CRL for this p12 certificate and CA
      //CA
      std::vector<Cryptonit::Certificate> CA;
      std::vector<Cryptonit::Certificate>::iterator it;
      CA = p12.getCA();
      for( it = CA.begin(); it != CA.end(); it++ )
      {
        std::string uri = it->extractCrlUri();
        if( uri != "" )
        {
          QString newCrlFile = AddCrlDlg::fetchCrl( uri.c_str() );
          Cryptonit::Crl newCrl;
          if( newCrl.load( newCrlFile.latin1()) == Cryptonit::SUCCESS )
          {
            Cryptonit::Certificate newCert( *it );
            AddCrlDlg::saveCrl( uri.c_str() , &newCrl , newCert , user );
          }
        }
      }
      //certificate
      std::string uri = cert.extractCrlUri();
      if( uri != "")
      {
        QString newCrlFile = AddCrlDlg::fetchCrl( uri.c_str() );
        Cryptonit::Crl newCrl;
        if( newCrl.load( newCrlFile.latin1()) == Cryptonit::SUCCESS )
        {
          AddCrlDlg::saveCrl( uri.c_str() , &newCrl , cert , user );
        }
      }

    }
  }
    //Importar a partir do token
  else if (rbToken->isOn())
  {
    tokenKeySelectionDlg *tkKeySelectionDlg = new tokenKeySelectionDlg(this);
    if( tkKeySelectionDlg->exec() == QDialog::Accepted )
    {
      int res;
      QString errorMsg;

      identity = tkKeySelectionDlg->getIdentity();
      if( identity.isEmpty())
        identity = "Default";

      // Reload the User with the updated values
      Cryptonit::Certificate *tokenCert = tkKeySelectionDlg->getCertificate();
      cert = *tokenCert;
      std::vector<Cryptonit::Certificate> *certChain = tkKeySelectionDlg->getChain();
      // Add identity to th user from the PKCS 12
      res = user->addIdentityFromToken( identity.latin1(),
                                      tkKeySelectionDlg->getKeyId(),
                                      cert,
				      certChain);
      identity.prepend("_");
      keyID = tkKeySelectionDlg->getKeyId();
      switch( res )
      {
      case 0:
        break;
      case -1: errorMsg = "No foi possvel carregar o PKCS#12.";
        break;
      case -2: errorMsg = "Este arquivo PKCS#12 j existe para esse usurio.";
        break;
      case -3: errorMsg = "Essa identidade j se encontra na lista, por favor utilize um nome diferente.";
        break;
      case -4: errorMsg = "Ocorreu um erro ao gravar certificado no disco.";
        break;
      case -5: errorMsg = "No foi possvel adicionar as informaes de identidade na lista de contatos.";
        break;
        case -6: errorMsg = "No foi possvel gravar em disco um certificado CA includo no PKCS#12.";
        break;
      case -7: errorMsg = "No foi possvel copiar o PKCS#12 no diretrio de usurio.";
        break;
      default: errorMsg = "Erro desconhecido.";
        break;
      }

      if(!errorMsg.isEmpty())
      {
        QMessageBox::critical(this, "Erro", errorMsg);
        return;
      }
    }
  }
  /* Add from a certificate */
  else
  {

    // Prompt the user for the Certificate file
    QString wildcard;
    wildcard = "Todos os formatos suportados (*.cer *.crt *.der *.pem);;"
               "Certificado (*.cer *.crt);;"
               "Certificado codificado em DER (*.der);;"
               "Certificado codificado em PEM (*.pem);;";

    QString certFilename = QFileDialog::getOpenFileName(
                             QDir::homeDirPath(),
                             wildcard,
                             this,
                             "open file dialog",
        "Escolha uma requisio de certificado assinada" );

    if( !certFilename.isEmpty())
    {
      Cryptonit::Certificate certificate;
      if (certificate.load( certFilename) != Cryptonit::SUCCESS)
      {
        QString msg;
        msg = "No foi possvel carregar certificado: ";
        msg += certFilename;
        QMessageBox::critical( this, "Erro", msg);
        accept();
        return;
      }

      QString requestsDir( user->getP12Dir().c_str() );

      QDir dir( requestsDir );
      if( !dir.isReadable())
      {
        QString msg;
        msg = "No foi possvel abrir seu diretrio P12 pessoal: ";
        msg += requestsDir;
        QMessageBox::critical( this, "Erro", msg);
        accept();
        return;
      }

      Cryptonit::pkcs10 request;
      //       bool match = false;

      QString filename;
      QString requestFilename;

      bool match = false;
      // Search in the P12 user dir, the certificate request
      // matching the given signed certificate
      QStringList entries = dir.entryList("*.p10", QDir::Files);
      QStringList::iterator it;
      for(it = entries.begin(); it != entries.end(); ++it)
      {
        requestFilename = *it;
        requestFilename.prepend( user->getP12Dir().c_str() );

        std::cout << (*it).latin1() << ": ";

        if( request.load( requestFilename ) == 0 )
        {
          if(request.matchCertificate( certificate ))
          {
            match = true;
            std::cout << "match !";
            break;
          }
        }
      }

      // We find the matching certificate request
      // Now we try to found the associated private key
      if( match )
      {
        entries = dir.entryList(QDir::Files);

        if( entries.count() == 0)
        {
          QMessageBox::critical(this, "Erro",
                                "No foi possvel encontrar uma chave privada associada a esse certificado.");
          accept();
          return;
        }

        QStringList::iterator it;
        for(it = entries.begin(); it != entries.end(); ++it)
        {
          QString privateKeyFilename( *it );
          privateKeyFilename.prepend( user->getP12Dir().c_str() );

          std::cout << "Key: " << (*it).latin1() << std::endl;

          // Ask for the private key password
          bool again = true;
          Cryptonit::pkcs8 privateKey;
          QString password;

          while( again )
          {
            PasswordDlg pd( this);

            if( pd.exec() == QDialog::Accepted)
            {
              password = pd.passwordField->text();

              if( privateKey.load( privateKeyFilename, Cryptonit::pem_format, password.latin1() ) == false )
              {
                QMessageBox::critical( this, "Erro",
                                       "No foi possvel abrir a chave privada. "
                                       "Senha incorreta.");
                again = true;
              }
              else
              {
                again = false;
              }
            }
            else
            { // The user hit the "Cancel" button
              accept();
              return;
            }
          }

          // Check if the private key match the certificate
          if( ! request.matchPrivateKey(privateKey) )
          {
            QMessageBox::critical( this, "Erro",
                                   "Chave privada no corresponde ao certificado.");
            accept();
            return;
          }

          // Ask for the identity, and check if it already exists
          QString identityName;
          again = true;
          std::vector<std::string> identities;
          identities = user->getInfos( "Identities" );

          while( again )
          {
            identityName = QInputDialog::getText("Identidade",
                                                 "Por favor, entre com um nome para a nova identidade. "
                                                 "Voc pode deixar esse campo em branco:");


            if( identityName.isEmpty() )
            {
              identityName = "Default";
            }
            else
            {
              // Aborting certificate importation
              QMessageBox::critical( this, "Erro",
                                     "Abortando importao de certificado.");
              accept();
              return;
            }

            // Check if the provided identity already exists
            if( std::find( identities.begin(), identities.end(), identityName.latin1() )
                != identities.end() )
            {
              // the identity is already in the list
              QMessageBox::critical( this, "Erro",
                                     "Essa identidade j existe, por favor escolha outra.");
              again = true;
            }
            else
            {
              again = false;
            }
          }

          int ret = user->addIdentityFromP10( identityName.latin1(),
                                              requestFilename.latin1(),
                                              certFilename.latin1(),
                                              privateKeyFilename.latin1(),
                                              password.latin1() );

          QString errorMsg;
          switch( ret )
          {
          case 0 : break;
          case -1: errorMsg = "No foi possvel carregar a chave privada gerada com a requisio de certificado.";
            break;
          case -2: errorMsg = "No foi possvel carregar a requisio de certificado de origem.";
            break;
          case -3: errorMsg = "A chave privada no corresponde  requisio de certificado.";
            break;
          case -4: errorMsg = "No foi possvel carregar o certificado.";
            break;
          case -5: errorMsg = "No foi possvel criar o objeto PKCS#12 intermedirio.";
            break;
          case -6: errorMsg = "No foi possvel criar arquivo temporrio para o objeto PKCS#12 em seu diretrio P12.";
            break;
          case -7: errorMsg = "No foi possvel escrever o PKCS#12 temporrio em seu diretrio P12.";
            break;
            // addIdentityFromP12 errors:
          case -51: errorMsg = "No foi possvel carregar o PKCS#12 gerado.";
            break;
          case -52: errorMsg = "Esse arquivo PKCS#12 j existe para esse usurio.";
            break;
          case -53: errorMsg = "Essa identidade j existe na lista, por favor use um nome diferente.";
            break;
          case -54: errorMsg = "Ocorreu um erro ao gravar o certificado no disco.";
            break;
          case -55: errorMsg = "No foi possvel adicionar informao de identidade na lista de contatos.";
            break;
          case -56: errorMsg = "No foi possvel gravar no disco um certificado CA includo no PKCS#12.";
            break;
          case -57: errorMsg = "No foi possvel copiar o PKCS#12 em seu diretrio de usurio.";
            break;
          default:  errorMsg = "Ocorreu um erro desconhecido.";
            errorMsg += "(";
            errorMsg += QString::number(ret) + ")";
            break;
          }

          if( !errorMsg.isEmpty())
          {
            QMessageBox::critical(this, "Erro", errorMsg);
            accept();
            return;
          }

          // The import was successfull, we can now remove
          // the temporary private key and request.

          // Remove privateKeyFilename
          std::cout << "Remove " << privateKeyFilename.latin1() << std::endl;
          QFile( privateKeyFilename ).remove();

          // Remove requestFilename
          std::cout << "Remove " << requestFilename.latin1() << std::endl;
          QFile( requestFilename ).remove();

          // set the global variables 'cert' and 'p12'
          cert.load(certFilename.latin1());

          std::string p12filename;
          p12filename =  user->getP12Dir();
          p12filename += cert.getHash();
          p12filename += ".p12";
          p12.load(p12filename.c_str(), password.latin1());


          //downloading CRL for this p12 certificate and CA
          //CA
          std::vector<Cryptonit::Certificate> CA;
          std::vector<Cryptonit::Certificate>::iterator it;
          CA = p12.getCA();
          for( it = CA.begin(); it != CA.end(); it++ )
          {
            std::string uri = it->extractCrlUri();
            if( uri != "" )
            {
              QString newCrlFile = AddCrlDlg::fetchCrl( uri.c_str() );
              Cryptonit::Crl newCrl;
              if( newCrl.load( newCrlFile.latin1()) == Cryptonit::SUCCESS )
              {
                Cryptonit::Certificate newCert( *it );
                AddCrlDlg::saveCrl( uri.c_str() , &newCrl , newCert , user );
              }
            }
          }
          //certificate
          std::string uri = cert.extractCrlUri();
          if( uri != "")
          {
            QString newCrlFile = AddCrlDlg::fetchCrl( uri.c_str() );
            Cryptonit::Crl newCrl;
            if( newCrl.load( newCrlFile.latin1()) == Cryptonit::SUCCESS )
            {
              AddCrlDlg::saveCrl( uri.c_str() , &newCrl , cert , user );
            }
          }

          accept();
          return;
        }

      }
      else
      {
        QMessageBox::critical(this, "Erro",
          "No h requisio de certificado no seu diretrio pessoal associado a esse certificado."),
        accept();
        return;
      }
    }
  }

  accept();
  return;
}


std::string AddIdentity::getKeyID()
{
	return keyID;
}
