/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you want to add, delete, or rename functions or slots, use
** Qt Designer to update this file, preserving your code.
**
** You should not define a constructor or destructor in this file.
** Instead, write your code in functions called init() and destroy().
** These will automatically be called by the form's constructor and
** destructor.
*****************************************************************************/

void AddressBookFrame::setInfo(User* u,std::vector<KeyStore *> *kstore) {
    user = u;
    keyStore = kstore;
    refreshContactList();
}

void AddressBookFrame::addBtn_clicked()
{
  enableButtons(false);
    
  QString wildcard;
      
  wildcard = QString("All supported formats (*.crt;*.cer;*.der;*.pem)\n")
       + QString("Certificate (*.crt;*.cer)\n")
       + QString("DER encoded certificate (*.der)\n")
       + QString("PEM encoded certificate (*.pem)\n");
     
  QString certFileName = QFileDialog::getOpenFileName(
                    QDir::homeDirPath(),
                    wildcard,
                    this,
                    QString("importa um certificado"),
                    QString("Escolha um arquivo de certificado para importar"));
    
  if( !certFileName.isEmpty()) {
      
     ContactDlg cd(this);
     bool userOk = false;
     userOk = cd.setUser(certFileName, user);
     if(userOk) { 
       if( cd.exec() == QDialog::Accepted){
        int ret;
        QString errorMsg = "";
        ret = cd.addContact(user, certFileName);

        switch( ret ) {
        case -1: errorMsg = QString("No foi possvel ler o certificado do contato ")+ 
          certFileName + QString(". Abortando.");
          break;
        case -2: errorMsg = QString("Ocorreu um erro na tentativa de gravar o certificado do contato no disco.");
          break;
        case -3: errorMsg = QString("Ocorreu um erro na tentativa de adicionar o contato: ");
          break;
        }

        if( !errorMsg.isEmpty()) {
          QMessageBox::critical(this, QString("Assinador"),  errorMsg);
        }

        // ask the user if he wants to download the new contact crl.
        Certificate newCert;
        if( newCert.load( certFileName.latin1() ) == SUCCESS ){
          std::string uri = newCert.extractCrlUri();
          if( uri != "" && Crl::outDated(uri , user , &newCert)){
            int answer = QMessageBox::question(this, "Assinador",  "Voc deseja efetuar o download da CRL deste certificado?", QMessageBox::Yes,  QMessageBox::No);
            if( answer == QMessageBox::Yes ){ 
          
               QString newCrlFile = AddCrlDlg::fetchCrl( uri.c_str() );
	   
     	       std::cout << "newCrlFile: " << newCrlFile << std::endl;
               std::cout << "Exists: " << QFile::exists(newCrlFile) << std::endl;
               Crl newCrl;
	     
    	       if( newCrl.load( newCrlFile.latin1()) == SUCCESS ){
	   	  std::cout << "crl loaded: "<< std::endl;
		  AddCrlDlg::saveCrl( uri.c_str() , &newCrl , newCert , user ); 
		  std::cout << "crl saved: "<< std::endl;
        
    	       } else {
        	  QMessageBox::critical(this, "Assinador",  "A CRL baixada  invlida.");
    	       }
	       
	       if (!newCrlFile.isEmpty()) {
	          //QFile removeme(newCrlFile);
	          //removeme.remove();
    	       }
	     }
          } else if(uri ==""){
             QMessageBox::information(this, "Assinador",  "O certificado deste contato no possui CRL.\n Adicione a CRL na tela de visualizao de certificado na aba 'CRL'.");
       
          }
      
        }
        refreshContactList();     
     }
   }  
 }
 enableButtons(true);
}


void AddressBookFrame::refreshContactList()
{
    contactList->clear();

    if( user->addressBook != NULL ) {
 std::vector<std::string> *names = user->addressBook->getAllContactName();
 std::vector<std::string>::iterator itr;
 long index;
 
 // Hide temporarly the list to speed up inserting
 contactList->hide();
 
 for( itr = names->begin(); itr != names->end(); itr++ ) {
     
     // Add the first field
   QListViewItem certItem = new QListViewItem( 
       contactList, 
       user->addressBook->getContactInfo(*itr, "cn").c_str(), 
       user->addressBook->getContactInfo(*itr, "mail").c_str(), 
       user->addressBook->getContactInfo(*itr, "sn").c_str());
 }

 contactList->show();
 removeBtn->setEnabled(true);
 // Fill the status bar with the number of contacts
 if( contactList->childCount () > 1 ) {
     statusBar->setText(QString(" ")+QString::number(contactList->childCount())+QString(" entradas"));
 }
 if( contactList->childCount () == 1 )
     statusBar->setText(QString(" ")+QString::number(contactList->childCount())+QString(" entrada"));
 
 if( contactList->childCount () == 0 ) {
     statusBar->setText(QString(" no existem contatos"));
     removeBtn->setEnabled(false);
 }    

 // Sort contacts using the first field
 contactList->setSortColumn (0);

 names->clear();
 delete names;
    }
}


void AddressBookFrame::contactList_doubleClicked(QListViewItem *item )
{
    QString name = item->text(0);
    
    std::vector<std::string> *names = user->addressBook->getAllContactName();
    std::vector<std::string>::iterator itr;

     for( itr = names->begin(); itr != names->end(); itr++ ) {               
     Entry *e = user->addressBook->getContactDSE(*itr);
     if(e->getAttributeFirstValue("cn") == name.latin1()) {
         ContactDlg cd(this);
	 
	 
  cd.setInfo(e, user);
      if(cd.exec() == QDialog::Accepted){
   cd.saveContact( user->addressBook->getContactDSE(*itr));
   refreshContactList();
      }
     }        
    }    
}


void AddressBookFrame::removeBtn_clicked()
{
   QString name("");
   if(contactList->selectedItem() == NULL) {
      name = contactList->currentItem()->text(0);
   }
   else {   
      name = contactList->selectedItem()->text(0);
   }
    
    std::vector<std::string> *names = user->addressBook->getAllContactName();
    std::vector<std::string>::iterator itr;

     for( itr = names->begin(); itr != names->end(); itr++ ) {               
     Entry *e = user->addressBook->getContactDSE(*itr);
     if(e->getAttributeFirstValue("cn") == name.latin1()) {
   /* Delete associated certificate(s) file(s) */
   QFile::remove(QString(user->addressBook->getContactInfo(*itr, "userCertificate").c_str()) );
       /* Remove the contact from the address book */
  user->addressBook->removeContact( *itr );
  refreshContactList();
     }        
    }    
}


void AddressBookFrame::enableButtons( bool enBtn )
{
   addBtn->setEnabled(enBtn);
   closeBtn->setEnabled(enBtn);
   
   if(enBtn == true) {
     removeBtn->setEnabled((contactList->childCount () != 0));
   }
   else {
     removeBtn->setEnabled(false);
   }   
}
