/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you want to add, delete, or rename functions or slots, use
** Qt Designer to update this file, preserving your code.
**
** You should not define a constructor or destructor in this file.
** Instead, write your code in functions called init() and destroy().
** These will automatically be called by the form's constructor and
** destructor.
*****************************************************************************/

#include "Utils.hh"
#include "Certificate.hh"
#include "pkcs12.hh"
#include "error.hh"

#include <qfiledialog.h>
#include <qdir.h>
#include <qmessagebox.h>

#include <iostream>

#include "PasswordDlg.h"

//QDesiger tem problemas com default arguments
//se salvar o CryptonitDlg.ui pelo QTDesiger, ele vai apagar a declaracao
//da funcao "init". Abaixo est a tag que fornece a declaracao no .ui:
//<function access="public" returnType="void">init(const bool askForIdentity = false, const QString defaultFilename = "", const QString defaultPassword = ""  )</function>
void Pkcs12Dlg::init(const bool askForIdentity,
                     const QString defaultFilename,
                     const QString defaultPassword)
{
  if(!askForIdentity)
  {
    identitySizerSB->hide();
  }

  pkcs12File->setText(defaultFilename);
  password = defaultPassword;

  if( defaultFilename != "" )
  {
    /* If the user entered a bad password, give
      * him a chance to reenter it.
    */
    int ret;
    do
    {
      ret = openPKCS12( defaultFilename );
      if( ret == -1 )
        password = "";
    }
    while( ret == -1 );
  }
}


QString Pkcs12Dlg::getPkcs12Filename()
{
  return pkcs12File->text();
}

QString Pkcs12Dlg::getPkcs12Password()
{
  return password;
}

QString Pkcs12Dlg::getIdentity()
{
  if( !identityName->text().isEmpty() )
    return identityName->text();
  else
    return QString::null;    
}

int Pkcs12Dlg::openPKCS12(QString filename)
{
  if( password == "" )
  {
    PasswordDlg pd(0, 0, true);
    if(pd.exec() == QDialog::Accepted )
    {
      password  = pd.passwordField->text();
    }
    else
    {
      return -2;
    }
  }

  //thiago: o cara nem se deu ao trabalho de colocar um #include para essa func.
  //ela deve ser da biblioteca do OpenSSL
  OpenSSL_add_all_algorithms();

  std::cerr << "Opening PKCS12: " << filename << std::endl;
  Cryptonit::pkcs12 *p12 = new Cryptonit::pkcs12();
  int error;

  if( (error = p12->load(filename.latin1(), password.latin1())) == Cryptonit::SUCCESS )
  {
    pkcs12File->setText(filename);
    Cryptonit::Certificate cert = p12->getCertificate();

    QString value = cert.getIssuerName().getValues(DN_DISPLAY_SHORT | DN_DISPLAY_VALUE , ',').c_str();
    lbIssuer->setText(value);
    value = cert.getSubjectName().getValues(DN_DISPLAY_SHORT | DN_DISPLAY_VALUE , ',').c_str();
    lbSubject->setText(value);

    value = cert.getSerialNumber().c_str();
    lbSerial->setText(value);

    delete p12;   //checker le destructeur de pkcs12 ...

    return 0;
  }
  else
  {
    QString msg = "Ocorreram os seguintes errors:\n";
    msg += Cryptonit::getErrorMsg(error).c_str();

    if(error == Cryptonit::PKCS12_PARSE_ERROR)
    {
      msg+= " \n";
      msg += "Senha invlida.\n";
    }

    QMessageBox::critical(this, "Erro", msg);

    std::cout << Cryptonit::getErrorMsg(error) << std::endl;

    return -1;
  }
}



void Pkcs12Dlg::slotOpenFile()
{
  QString wildcard;
  wildcard = "Arquivo PKCS#12 (*.p12;*.pkcs12;*.pfx)";

  QString filename = QFileDialog::getOpenFileName(
                       QDir::homeDirPath (),
                       wildcard,
                       this,
                       "open file dialog",
         "Escolha um arquivo PKCS#12 para importar" );

  if(!filename.isEmpty())
  {

    /* The user entered a bad password, give hime
      * a chance to reenter it.
    */
    int ret;
    do
    {
      ret = openPKCS12( filename.latin1() );
      if( ret == -1 )
        password = "";
    }
    while( ret == -1 );
  }
}
