/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you want to add, delete, or rename functions or slots, use
** Qt Designer to update this file, preserving your code.
**
** You should not define a constructor or destructor in this file.
** Instead, write your code in functions called init() and destroy().
** These will automatically be called by the form's constructor and
** destructor.
*****************************************************************************/
#include <qdir.h>

#include "../Contact.hh"
#include "../Certificate.hh"
#include "GenericDataListViewItem.h"
using namespace Cryptonit;

void RecipientsDlg::addRecipient(QListViewItem *item)
{
  availableRecipients->takeItem(item);
  selectedRecipients->insertItem(item);
  if(signer)
    add->setEnabled(FALSE);
  else
    add->setEnabled(availableRecipients->childCount()>0);
  remove->setEnabled(TRUE);
}



void RecipientsDlg::setSigner( bool s )
{
  QTextCodec *codec = QTextCodec::codecForName("utf-8");
  signer = s;
  if(signer)
  {
    mainBox->setTitle("Escolha o assinante destes arquivos");
    availableBox->setTitle(codec->toUnicode("Assinantes disponveis"));
    selectedBox->setTitle("Assinantes selecionados");
  }
  else
  {
    mainBox->setTitle(codec->toUnicode("Escolha os destinatrios destes arquivos"));
    availableBox->setTitle(codec->toUnicode("Destinatrios disponveis"));
    selectedBox->setTitle(codec->toUnicode("Destinatrios selecionados"));
  }
}

int RecipientsDlg::exec()
{
std::cout << "exec() " << std::endl;
  if(signer)
    fillAvailableSigners();
  else
    fillAvailableRecipients();
  return QDialog::exec();
}

void RecipientsDlg::init()
{
  signer=false;
}


void RecipientsDlg::fillAvailableRecipients()
{
std::cout << "fillAvailableRecipients" << std::endl;
  if( user->addressBook != NULL )
  {
    std::vector<std::string>*names = user->addressBook->getAllContactName();
    std::vector<std::string>::iterator itr;
    long i = 0;

    for( itr = names->begin(); itr != names->end(); itr++ )
    {
      std::cout << "*names: " << *itr << std::endl;
      
      Certificate c;
      std::string certFile = user->getCertificatesDir() + QDir::separator() + user->addressBook->getContactInfo(*itr, "userCertificate");
            
      std::cout << "certFile: " << certFile << std::endl;
      
      if(c.load(certFile.c_str()) == SUCCESS)
      {
        if(c.isForEncrypting() ||  c.isForSigningAndEncrypting())
        {
          std::string *data = new std::string( *itr );
          std::string aux = user->addressBook->getContactInfo(*itr, "cn");
          new GenericDataListViewItem(availableRecipients, (void*)data, QString(aux.c_str()));
          add->setEnabled(TRUE);
          i++;
        }
      }
    }

    /* Sort recipients, using wxLC_SORT_ flag */
    availableRecipients->sort();

    names->clear();
    delete names;

  }

}





void RecipientsDlg::destroy()
{

  QListViewItemIterator *it = new QListViewItemIterator(availableRecipients);
  while ( it->current() )
  {
    GenericDataListViewItem *item = (GenericDataListViewItem*)it->current();
    std::string *data = (std::string*) item->getData();
    if( data ) delete data;
    ++(*it);
  }
  delete it;
  it = new QListViewItemIterator(selectedRecipients);
  while ( it->current() )
  {
    GenericDataListViewItem *item = (GenericDataListViewItem*)it->current();
    std::string *data = (std::string*) item->getData();
    if( data ) delete data;
    ++(*it);
  }
  delete it;
}


void RecipientsDlg::fillAvailableSigners()
{
std::cout << "fillAvailableSigners" << std::endl;
  if( user->addressBook != NULL )
  {
    std::vector<std::string>* names = user->addressBook->getAllContactName();
    std::vector<std::string>::iterator itr;
    long i = 0;

    for( itr = names->begin(); itr != names->end(); itr++ )
    {
      Certificate c;
      std::string certFile = user->getCertificatesDir() + QDir::separator() + user->addressBook->getContactInfo(*itr, "userCertificate");
      if(c.load(certFile.c_str()) == SUCCESS)
      {
        if(c.isForSigning() ||  c.isForSigningAndEncrypting())
        {
          std::string *data = new std::string( *itr );
          GenericDataListViewItem item(availableRecipients, data, QString(user->addressBook->getContactInfo(*itr, "cn").c_str()));
          i++;
        }
      }
    }
    /* Sort recipients, using wxLC_SORT_ flag */
    availableRecipients->sort();
    names->clear();
    delete names;
  }

}


void RecipientsDlg::setUser( Cryptonit::User *_user )
{
  user = _user;
}


void RecipientsDlg::enableAddBtn( QListViewItem *item )
{
  bool enable = (item != NULL);
  if(signer)
    add->setEnabled(enable & (selectedRecipients->childCount()==0));
  else
    add->setEnabled(enable);

}


void RecipientsDlg::enableRemoveBtn( QListViewItem *item )
{
  remove->setEnabled((item != NULL));
}


void RecipientsDlg::removeRecipient(QListViewItem *item)
{
  selectedRecipients->takeItem(item);
  availableRecipients->insertItem(item);
  if(signer)
    remove->setEnabled(FALSE);
  else
    remove->setEnabled(selectedRecipients->childCount()>0);
  add->setEnabled(TRUE);
}


void RecipientsDlg::removeBtn_clicked()
{
  QListViewItem *item = selectedRecipients->currentItem();
  removeRecipient(item);
}


void RecipientsDlg::addBtn_clicked()
{
  QListViewItem *item = availableRecipients->currentItem();
  addRecipient(item);
}


std::vector<QString>* RecipientsDlg::getRecipientList()
{
  if( selectedRecipients->childCount() == 0 )
    return NULL;

  std::vector<QString>* result = new std::vector<QString>();
  QListViewItemIterator *it = new QListViewItemIterator(selectedRecipients);
  while ( it->current() )
  {
    GenericDataListViewItem *item = (GenericDataListViewItem*)it->current();
    std::cout << item->text(0) << std::endl;
    std::string *name = (std::string*) item->getData();
    if(name == NULL)
    {
      std::cout << "data == NULL" << std::endl;
    }
    else
    {
      std::cout << *name << std::endl;
      result->push_back(QString(name->c_str()));
    }
    ++(*it);
  }
  return result;
}


QString RecipientsDlg::getRecipient()
{
  if( selectedRecipients->childCount() == 0 )
    return QString("");
  GenericDataListViewItem *item = (GenericDataListViewItem*)selectedRecipients->firstChild();
  std::string *name = (std::string *)item->getData();

  return QString(name->c_str());

}
