/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you want to add, delete, or rename functions or slots, use
** Qt Designer to update this file, preserving your code.
**
** You should not define a constructor or destructor in this file.
** Instead, write your code in functions called init() and destroy().
** These will automatically be called by the form's constructor and
** destructor.
*****************************************************************************/

#include <iostream>;
#include <qradiobutton.h>
#include <qmessagebox.h>
#include <qapplication.h>
#include <qcursor.h>
#include <qthread.h>

#include "tokenkeystore.h"
#include "Certificate.hh"
#include "CertificateViewer.h"

void tokenKeySelectionDlg::init()
{
  certChain = NULL;
  std::cout << __FUNCTION__ << std::endl;
  keyGroup->setTitle("Lendo contedo do token...");
  rbOne->hide();
  rbTwo->hide();
  rbThree->hide();
  report = "";
  listKeyProcess = new QProcess(this);
  listKeyProcess->addArgument("pkcs15-tool");
  listKeyProcess->addArgument("-k");
  connect( listKeyProcess, SIGNAL(readyReadStdout()), this, SLOT(readKeysOutput()) );
  connect( listKeyProcess, SIGNAL(processExited()), this, SLOT(KeyReadingFinished()) );
  if (!listKeyProcess->launch(QString::null)) {
    delete listKeyProcess;
  }
  std::cout << __FUNCTION__ << " END" << std::endl;
}


void tokenKeySelectionDlg::readKeysOutput()
{
  std::cout << __FUNCTION__ << std::endl;
  QString line;
  while (!(line = listKeyProcess->readLineStdout()).isNull())
  {
    report += line;
    report += "\n";
  }
  std::cout << __FUNCTION__ << " END" << std::endl;
}


void tokenKeySelectionDlg::viewCert()
{
  std::cout << __FUNCTION__ << std::endl;
  Cryptonit::Certificate *cert = getCertificate();
  CertificateViewer cv(this);
  cv.init(*cert, NULL, false);
  cv.exec();
  std::cout << __FUNCTION__ << " END" << std::endl;
}


void tokenKeySelectionDlg::KeyReadingFinished()
{
  std::cout << __FUNCTION__ << " normalExit: " << listKeyProcess->normalExit() << ", exitStatus: " <<
  listKeyProcess->exitStatus() << std::endl;
  //Libera memria de todos os certificados que foram lidos do smartcard.
  std::vector< std::pair< std::string, Cryptonit::Certificate* > >::iterator tokenCertsIt;
  for( tokenCertsIt = tokenCerts.begin() ; tokenCertsIt != tokenCerts.end() ; tokenCertsIt++) {
  	tokenCerts.erase(tokenCertsIt);
  }
  tokenCerts.clear();
  keyIDs.clear();
  certIDs.clear();
  if (certChain != NULL) {
  	certChain->clear();
	delete certChain;
	certChain = NULL;
  }
  if (listKeyProcess->normalExit() && listKeyProcess->exitStatus() == 0)
  {
    bool keyFound = false;
    if (!report.isEmpty())
    {
      keyIDs = TokenKeyStore::listKeys(report);
      if (keyIDs.size() > 0) {
        keyFound = true;
        report = "";
        listCertsProcess = new QProcess(this);
        listCertsProcess->addArgument("pkcs15-tool");
        listCertsProcess->addArgument("-c");
        connect( listCertsProcess, SIGNAL(readyReadStdout()), this, SLOT(readCertsOutput()) );
        connect( listCertsProcess, SIGNAL(processExited()), this, SLOT(certsReadingFinished()) );
        if (!listCertsProcess->launch(QString::null)) {
	  delete listCertsProcess;
	}
      }
    }
    if (!keyFound)
    {
      keyGroup->setTitle("Token no contm chaves privadas vlidas");
    }
  }
  else
  {
    keyGroup->setTitle("Falha no acesso ao token");
  }
  delete listKeyProcess;
  std::cout << __FUNCTION__ << " END" << std::endl;
}


void tokenKeySelectionDlg::readCertOutput()
{
  std::cout << __FUNCTION__ << std::endl;
  QString line;
  while (!(line = readCertProcess->readLineStdout()).isNull())
  {
    std::cout << line << std::endl;
    report += line;
    report += "\n";
  }
  std::cout << __FUNCTION__ << " END" << std::endl;
}


void tokenKeySelectionDlg::certReadingFinished()
{
  std::cout << __FUNCTION__ << " normalExit: " << readCertProcess->normalExit() << ", exitStatus: " <<
  readCertProcess->exitStatus() << std::endl;
  if (readCertProcess->normalExit() && readCertProcess->exitStatus() == 0)
  {
    if (!report.isEmpty())
    {
      char temp[] = "/tmp/assinadorXXXXXX";
      int fd = mkstemp(temp);
      QFile file(temp);
      if ( file.open( IO_WriteOnly ) )
      {
        QTextStream stream( &file );
        stream << report;
        file.close();
        std::cout << "Certificado salvo no arquivo: " << temp << std::endl;
        tempCert = TokenKeyStore::decodeCert(std::string(tempId), temp);
        //TODO: Apagar o arquivo temporrio
	file.remove();
	if (tempCert != NULL) {
	  std::pair< std::string, Cryptonit::Certificate* > idAndCert(tempId, tempCert);
	  tokenCerts.push_back(idAndCert);
	  totalIdRead++;
	  tryReadCert();
	}
      }
      report = "";
    }
  }
  QApplication::restoreOverrideCursor();
  //delete readCertProcess;
  std::cout << __FUNCTION__ << " END" << std::endl;
}


std::string tokenKeySelectionDlg::getKeyId()
{
  return std::string(keyGroup->selected()->name());
}


Cryptonit::Certificate* tokenKeySelectionDlg::getCertificate()
{
  return tempCert;
}


QString tokenKeySelectionDlg::getIdentity()
{
  return identityName->text();
}


void tokenKeySelectionDlg::enableViewCert( bool enabled )
{
  std::cout << __FUNCTION__ << std::endl;
  if (enabled)
  {
    std::string keyID = std::string(keyGroup->selected()->name());
    for (int j=0; j<tokenCerts.size(); j++) {
      std::cout << "Id do Certificado : " << tokenCerts[j].first << std::endl;
      if (tokenCerts[j].first == keyID) {
        if (certChain != NULL) {
		certChain->clear();
		delete certChain;
		certChain = NULL;
	}
	tempCert = tokenCerts[j].second;
      }
    }
    std::cout << keyGroup->selected()->name() << std::endl;
    viewCertBtn->setEnabled(TRUE);
    okBtn->setEnabled(TRUE);
  }
  std::cout << __FUNCTION__ << " END" << std::endl;
}


void tokenKeySelectionDlg::accept()
{
  std::cout << __FUNCTION__ << std::endl;
  if (identityName->text().isEmpty())
  {
    QMessageBox::critical(this, "Erro", "Nome da nova identidade  obrigatrio.");
  }
  else
  {
    QDialog::accept();
  }
  std::cout << __FUNCTION__ << " END" << std::endl;
}


void tokenKeySelectionDlg::readCertsOutput()
{
  std::cout << __FUNCTION__ << std::endl;
  QString line;
  while (!(line = listCertsProcess->readLineStdout()).isNull())
  {
    std::cout << line << std::endl;
    report += line;
    report += "\n";
  }
  std::cout << __FUNCTION__ << " END" << std::endl;
}


void tokenKeySelectionDlg::certsReadingFinished()
{
  std::cout << __FUNCTION__ << " normalExit: " << listCertsProcess->normalExit() << ", exitStatus: " <<
  listCertsProcess->exitStatus() << std::endl;
  if (listCertsProcess->normalExit() && listCertsProcess->exitStatus() == 0)
  {
    if (!report.isEmpty())
    {
      std::cout << report.latin1() << std::endl;
      totalIdRead = 0;
      certIDs = TokenKeyStore::listCerts(report);
      std::cout << "Quantidade de CertIds: " << certIDs.size() << std::endl;
      if (certIDs.size() > 0) {
      	tryReadCert();
      }
    }
  }
  else
  {
    keyGroup->setTitle("Falha no acesso ao token (leitura de Certificados)");
  }
  delete listCertsProcess;
  std::cout << __FUNCTION__ << " END" << std::endl;
}


void tokenKeySelectionDlg::fillKeyOptions()
{
	if (readCertProcess != NULL) {
		delete readCertProcess;
		readCertProcess = NULL;
	}
  std::cout << __FUNCTION__ << std::endl;
      std::cout << "Quantidade de certificados lidos: " << tokenCerts.size() << std::endl;
      if (keyIDs.size() > 0)
      {
        for (int i=0; i<keyIDs.size() && i<3; i++)
        {
	  std::cout << "KeyID : " << keyIDs[i] << std::endl;
	  for (int j=0; j<tokenCerts.size(); j++) {
	    std::cout << "CertID : " << tokenCerts[j].first << std::endl;
	    if (tokenCerts[j].first == keyIDs[i]) {
   	      QRadioButton *item = (QRadioButton*)keyGroup->find(i);
	      item->setName(QString(keyIDs[i]));
	      item->setText(QString((tokenCerts[j].second)->getSubjectName().getValue("CN")));
	      item->show();
	    }
	  }
        }
        keyGroup->setTitle("");
      }
  std::cout << __FUNCTION__ << " END" << std::endl;
}


void tokenKeySelectionDlg::tryReadCert()
{
  std::cout << __FUNCTION__ << std::endl;
  std::cout << "totalIdRead: " << totalIdRead << ", certIDs.size(): " << certIDs.size() << std::endl;
  if ((totalIdRead >= 0) && (totalIdRead < certIDs.size())) {
	report = "";
	tempId = certIDs[totalIdRead];
	tempCert = NULL;
	if (readCertProcess != NULL) {
		delete readCertProcess;
		readCertProcess = NULL;
	}
	readCertProcess = new QProcess(this);
	readCertProcess->addArgument("pkcs15-tool");
	readCertProcess->addArgument("-r");
	readCertProcess->addArgument(tempId);
	connect( readCertProcess, SIGNAL(readyReadStdout()), this, SLOT(readCertOutput()) );
	connect( readCertProcess, SIGNAL(processExited()), this, SLOT(certReadingFinished()) );
	if (!readCertProcess->launch(QString::null)) {
	  delete readCertProcess;
	}
  }
  if (totalIdRead == certIDs.size()) {
    fillKeyOptions();
    
  }
  std::cout << __FUNCTION__ << " END" << std::endl;
}


std::vector<Cryptonit::Certificate>* tokenKeySelectionDlg::getChain()
{
	std::vector<Cryptonit::Certificate> *chain = new std::vector<Cryptonit::Certificate>(0);
	if (certChain == NULL && tempCert != NULL) {
		std::vector<Certificate> *certList = new std::vector<Cryptonit::Certificate>;
  		std::vector<std::string> idList;
  		for (int j=0; j<tokenCerts.size(); j++) {
			Cryptonit::Certificate* cert = tokenCerts[j].second;
			certList->push_back(*cert);
  		}
  		certChain = tempCert->getChain(certList);
		certList->clear();
  		delete certList;
	}
	if (certChain != NULL) {
		chain = certChain;
	}
	return chain;
}
