/*
 * The Cryptonit security software suite is developped by IDEALX
 * Cryptonit Team (http://IDEALX.org/ and http://cryptonit.org).
 *
 * Copyright ? 2003 IDEALX
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */


#include "pkcs7.hh"


namespace Cryptonit {

    pkcs7::pkcs7()
    {
	//certs = sk_X509_new_null();
	p7 = PKCS7_new();
	p7bio=NULL;
	cipher = NULL;
	digest = NULL;
	isLoaded = false;
    }

    pkcs7::~pkcs7()
    {
	PKCS7_free(p7);
	certs.clear();
    }

    int pkcs7::setContentType(PKCS7ContentType cType)
    {
	if(p7) {
	    if(cType == PKCS7Data || cType ==  PKCS7Signed ||
	       cType == PKCS7Encrypted || cType == PKCS7SignedEncrypted) {
		return(PKCS7_set_type(p7,cType));
	    } else {
		return(PKCS7_set_type(p7,NID_undef));
	    }
	}
	return 0;
    }


    PKCS7ContentType pkcs7::getContentType()
    {
	return (PKCS7ContentType)OBJ_obj2nid(p7->type);
    }


    int pkcs7::setCipher(CipherName name)
    {
	if(p7) {
	    switch(name) {
#ifndef OPENSSL_NO_DES
	    case C_DES:
		cipher = EVP_des_cbc();
		break;
	
	    case C_DES3:
		cipher = EVP_des_ede3_cbc();
		break;
#endif
#ifndef OPENSSL_NO_RC2     
	    case C_RC2_128:
		cipher = EVP_rc2_cbc();
		break;
      
	    case C_RC2_64:
		cipher = EVP_rc2_64_cbc();
		break;
      
	    case C_RC2_40:
		cipher = EVP_rc2_40_cbc();
		break;
#endif
#ifndef OPENSSL_NO_AES      
	    case C_AES_128:
		cipher = EVP_aes_128_cbc();
		break;

	    case C_AES_192:
		cipher = EVP_aes_192_cbc();
		break;

	    case C_AES_256:
		cipher = EVP_aes_256_cbc();
		break;
#endif	

	    default:
		return 0;
		break;
	    }
	    return PKCS7_set_cipher(p7,cipher);
	}
	return 0;
    }




    int pkcs7::setCipher(std::string name)
    {
	bool ok = false;
	if(p7) {
	    
#ifndef OPENSSL_NO_DES
	    if(name == "DES"){
		cipher = EVP_des_cbc();
		ok = true;
	    }
	    
	    if (name == "DES3"){
		cipher = EVP_des_ede3_cbc();
		ok = true;
	    }

	    if (name == "DESX"){
		cipher = EVP_desx_cbc();
		ok = true;
	    }
#endif
#ifndef OPENSSL_NO_RC2     
	    if (name == "RC2_128"){
	      cipher = EVP_rc2_cbc();
		ok = true;
	    }

      	    if (name == "RC2_64"){
		cipher = EVP_rc2_64_cbc();
		ok = true;
	    }
	    
	    if (name == "RC2_40"){
		cipher = EVP_rc2_40_cbc();
		ok = true;
	    }
#endif
#ifndef OPENSSL_NO_RC4
	    if(name == "RC4"){
		cipher = EVP_rc4();
		ok = true;
	    }
#endif
#ifndef OPENSSL_NO_RC5
	    if(name == "RC5"){
		cipher = EVP_rc5_32_12_16_cbc();
		ok = true;
	    }   
#endif
#ifndef OPENSSL_NO_IDEA
	    if(name == "IDEA"){
		cipher = EVP_idea_cbc();
		ok = true;
	    }   	  
#endif
#ifndef OPENSSL_NO_BF
	    if(name == "BF"){
		cipher = EVP_bf_cbc();
		ok = true;
	    }
#endif
#ifndef OPENSSL_NO_CAST
	    if( name == "CAST"){
		cipher = EVP_cast5_cbc();
		ok = true;
	    }
	    
#endif
#ifndef OPENSSL_NO_AES      
	  if (name == "AES_128"){
	      cipher = EVP_aes_128_cbc();
	      ok = true;
	  }
	  
	  if(name == "AES_192"){
	      cipher = EVP_aes_192_cbc();
	      ok = true;
	  }
	  
	  if( name =="AES_256"){
	      cipher = EVP_aes_256_cbc();
	      ok = true;
	  }
#endif	
	  if(ok)
	      return PKCS7_set_cipher(p7,cipher);
	}
	return 0;
    }


  

    void pkcs7::setDigest(DigestName name)
    {
	switch(name) {
	case D_MD2:
	    digest = EVP_md2();
	    break;
	case D_MD5:
	    digest = EVP_md5();
	    break;
	case D_SHA1:
	    digest = EVP_sha1();
	    break;
	default:
	    break;
	}
    }



    void pkcs7::setDigest(std::string name)
    {
#ifndef OPENSSL_NO_MD2
	if (name == "MD2") {
	    digest = EVP_md2();
	    return;
	}
#endif
#ifndef OPENSSL_NO_MD4 	
	if (name == "MD4") {
	    digest = EVP_md4();
	    return;
	}
#endif
#ifndef OPENSSL_NO_MD5	
	if (name =="MD5") {
	    digest = EVP_md5();
	    return;
	}
#endif
#ifndef OPENSSL_NO_SHA
	if (name == "SHA1") {
	    digest =  EVP_sha1();
	    return;
	}
#endif	
#if !defined(OPENSSL_NO_MDC2) && !defined(OPENSSL_NO_DES)
	if (name == "MDC2") {
	    digest = EVP_mdc2();
	    return;
	}
#endif
#ifndef OPENSSL_NO_RIPEMD	
	if (name == "RIPEMD" ) {
	    digest = EVP_ripemd160();
	    return;
	}
#endif
	
    }



  
    bool pkcs7::write(const char *fileout, FileFormat format)
    {
	if( getContentType() == PKCS7Signed 
	    || getContentType() == PKCS7Encrypted 
	    || getContentType() == PKCS7SignedEncrypted ) {
	    
      
	    BIO *out;
	    if( !(out = BIO_new_file(fileout, "wb")) ) {
		std::cout << getErrorMsg(PKCS7_BIO_NEW_ERROR) << std::endl;
		return PKCS7_BIO_NEW_ERROR;
	    }
	    
	    if( format == pem_format ) {
	    
		if( PEM_write_bio_PKCS7(out,p7) ) {
		    BIO_free_all(out);
		    
		    return SUCCESS;	
		} else {
		    BIO_free_all(out);
		    
		    std::cout << getErrorMsg(PKCS7_BIO_WRITE_ERROR) << std::endl;
		    return PKCS7_BIO_WRITE_ERROR;
		}
		
	    }
	    else if( format == der_format ) {
	    
		if( i2d_PKCS7_bio(out,p7) ) {
		    BIO_free_all(out);		    
		    return SUCCESS;
		} else {
		    BIO_free_all(out);		    
		    std::cout << getErrorMsg(PKCS7_BIO_WRITE_ERROR) << std::endl;
		    return PKCS7_BIO_WRITE_ERROR;
		}
	    }
	    
	    std::cout << getErrorMsg(PKCS7_BAD_FORMAT) << std::endl;
	    return PKCS7_BAD_FORMAT;
	}
	
	std::cout << getErrorMsg(PKCS7_BAD_CONTENT_TYPE) << std::endl;
	return PKCS7_BAD_CONTENT_TYPE;
    }

    
    int pkcs7::writeDecrypted(const char *fileout)
    {
	FILE *fout;
	unsigned char buf[65535];
	
	fout = fopen( fileout , "wb" );
	
	if(fout) {
	    int i = 0;
	    while( true ) {
		i = BIO_read(p7bio, buf, sizeof(buf));
		if (i <= 0) break;
		fwrite(buf,  sizeof(char), i, fout);
	    }
	    fclose(fout);
	    BIO_free(p7bio);
	    p7bio = NULL;
	    return SUCCESS;
	}
	return  PKCS7_FILE_CREATE_ERROR;
    }
  

    int pkcs7::read(const char *filein, FileFormat format)
    {
	isLoaded = false;
	FILE *fin;
	fin=fopen(filein, "r" );
    
	if(fin) {
	    if(p7) {
		PKCS7_free(p7);
		p7 = NULL;
	    }
      
	    if( format==pem_format )
		p7 = PEM_read_PKCS7(fin, NULL, NULL, NULL);
	    else if (format==der_format)
		p7=d2i_PKCS7_fp(fin,NULL);
      
	    fclose(fin);
	    if(p7) {
	        isLoaded = true;
		return SUCCESS;
	    }
      
	    return PKCS7_READ_ERROR;
	}
	else
	    return PKCS7_FILE_NOT_FOUND;
    }
  


    int pkcs7::read(const char *filein)
    {
	if(read(filein, pem_format) != SUCCESS)
	    if(read(filein, der_format) != SUCCESS)
		return PKCS7_READ_ERROR;
	return SUCCESS;
    }

  
    int pkcs7::addRecipient(Certificate &recipient)
    {
	if(p7) {
	    //      PKCS7_add_recipient(p7, recipient.getX509Certificate());
	    //sk_X509_push(certs, recipient.getX509Certificate());
	    certs.push_back(recipient);
	    return 1;
	}
	return 0;
    }
  
  
    int pkcs7::addRecipients(std::vector<Certificate> &recipients)
    {
	int i = 0;
	if(p7) {
	    i=1;
	    for( std::vector<Certificate>::iterator it = recipients.begin() ; it != recipients.end() ; it++ )
		i += addRecipient(*it);
	}
	return i;
    }
  

    int pkcs7::encrypt(const char *filein)
    {
	BIO *in;

	if (!(in = BIO_new_file(filein, "r")))
	    return  PKCS7_BIO_NEW_ERROR;

	std::vector<Certificate>::iterator it;
	
	STACK_OF(X509) *stack = sk_X509_new_null();
    
	for(it = certs.begin(); it != certs.end() ; it++) {
	    sk_X509_push(stack,  it->getX509Certificate());
	}
    
	if((p7 = PKCS7_encrypt( stack, in ,cipher , PKCS7_BINARY  )) == NULL)
	    return PKCS7_ENCRYPT_ERROR;
    
	sk_X509_pop_free(stack, X509_free);
	
	BIO_free(in);
	
	return SUCCESS;    
    }

  
    int pkcs7::decrypt(Key *pkey, Certificate *cert)
    {
	return ((p7bio = PKCS7_dataDecode(p7, pkey->getPrivateKey(), NULL, 
					  cert->getX509Certificate())) == NULL) ?
	    PKCS7_DECRYPT_ERROR : SUCCESS;
    }

    int pkcs7::sign(const char *filein, Certificate &cert, Key &privateKey,
		    std::vector<Certificate> &certChain, bool detached)
    {
	//Tenta carregar o arquivo como um pkcs7. 
	//Caso ok a variavel isLoaded  setada para true pelo metodo read.
	read(filein);
	
	
	STACK_OF(509) *chain=NULL;
	std::vector<Certificate>::iterator certIt;
	X509 *certif = cert.getX509Certificate();
	EVP_PKEY *pkey = privateKey.getPrivateKey();
	BIO *in;
	int flag = PKCS7_BINARY;
	p7bio = NULL;
    

	if(!X509_check_private_key(certif, pkey)) {
	    PKCS7err(PKCS7_F_PKCS7_SIGN,PKCS7_R_PRIVATE_KEY_DOES_NOT_MATCH_CERTIFICATE);
	    return PKCS7_PKEY_CERT_MISMATCH;
	}

	//Caso seja um pkcs7 carrega o BIO do envelope.
	if(isLoaded) {

	   if(!(in = PKCS7_dataInit(p7, NULL))) {	       
	       return PKCS7_BIO_NEW_ERROR;
	   }

	}
	else {

	   //Caso contrario cria um novo BIO a partir do arquivo.
	   if (!(in = BIO_new_file(filein, "r"))) {
	       return PKCS7_BIO_NEW_ERROR;
	   }    

	}    
	
	
	if( (certif != NULL) && (pkey !=NULL) ){
	    chain = sk_X509_new_null();
	    for(certIt=certChain.begin(); certIt!=certChain.end(); certIt++)
		sk_X509_push(chain, certIt->getX509Certificate());
	    
	    if(detached)
		flag = PKCS7_BINARY | PKCS7_DETACHED;
	
		
	    if(!(p7 = my_PKCS7_sign(certif,pkey, chain, in, flag ))){
		return PKCS7_SIGN_ERROR;
	    }
	    
	    return SUCCESS;
	}
	return PKCS7_CERT_OR_KEY_NULL;
    }
  
  

    PKCS7 *pkcs7::my_PKCS7_sign(X509 *signcert, EVP_PKEY *pkey, STACK_OF(X509) *certs,
				BIO *data, int flags)
    {
	PKCS7 *p7Aux;
	PKCS7_SIGNER_INFO *si;
	BIO *p7bio;
	STACK_OF(X509_ALGOR) *smcap;
	int i;
	
	if(!X509_check_private_key(signcert, pkey)) {
	    PKCS7err(PKCS7_F_PKCS7_SIGN,PKCS7_R_PRIVATE_KEY_DOES_NOT_MATCH_CERTIFICATE);
	    return NULL;
	}
	
	if(!(p7Aux = PKCS7_new())) {
	    PKCS7err(PKCS7_F_PKCS7_SIGN,ERR_R_MALLOC_FAILURE);
	    return NULL;
	}
	
	PKCS7_set_type(p7Aux, NID_pkcs7_signed);
	
	PKCS7_content_new(p7Aux, NID_pkcs7_data);
	
	if(digest == NULL)
	    setDigest("SHA1");
	
    	if (!(si = PKCS7_add_signature(p7Aux,signcert,pkey,digest))) {
		PKCS7err(PKCS7_F_PKCS7_SIGN,PKCS7_R_PKCS7_ADD_SIGNATURE_ERROR);
		return NULL;
	}
	
	if(!(flags & PKCS7_NOCERTS)) {
	    PKCS7_add_certificate(p7Aux, signcert);
	    if(certs) for(i = 0; i < sk_X509_num(certs); i++)
		PKCS7_add_certificate(p7Aux, sk_X509_value(certs, i));
	}
	
	if(!(p7bio = PKCS7_dataInit(p7Aux, NULL))) {
	    PKCS7err(PKCS7_F_PKCS7_SIGN,ERR_R_MALLOC_FAILURE);
	    return NULL;
	}
		
	SMIME_crlf_copy(data, p7bio, flags);
	
	if(!(flags & PKCS7_NOATTR)) {
	    PKCS7_add_signed_attribute(si, NID_pkcs9_contentType,
				       V_ASN1_OBJECT, OBJ_nid2obj(NID_pkcs7_data));
	    /* Add SMIMECapabilities */
	    if(!(flags & PKCS7_NOSMIMECAP))
		{
		    if(!(smcap = sk_X509_ALGOR_new_null())) {
			PKCS7err(PKCS7_F_PKCS7_SIGN,ERR_R_MALLOC_FAILURE);
			return NULL;
		}
#ifndef OPENSSL_NO_DES
		    PKCS7_simple_smimecap (smcap, NID_des_ede3_cbc, -1);
#endif
#ifndef OPENSSL_NO_RC2
		    PKCS7_simple_smimecap (smcap, NID_rc2_cbc, 128);
		    PKCS7_simple_smimecap (smcap, NID_rc2_cbc, 64);
#endif
#ifndef OPENSSL_NO_DES
		    PKCS7_simple_smimecap (smcap, NID_des_cbc, -1);
#endif
#ifndef OPENSSL_NO_RC2
		    PKCS7_simple_smimecap (smcap, NID_rc2_cbc, 40);
#endif
		    PKCS7_add_attrib_smimecap (si, smcap);
		    sk_X509_ALGOR_pop_free(smcap, X509_ALGOR_free);
		}
	}
	
	if(flags & PKCS7_DETACHED)PKCS7_set_detached(p7Aux, 1);
	
        if (!PKCS7_dataFinal(p7Aux,p7bio)) {
	    PKCS7err(PKCS7_F_PKCS7_SIGN,PKCS7_R_PKCS7_DATASIGN);
	    return NULL;
	}
	
        BIO_free_all(p7bio);
	
	//Caso seja um pkcs7 novo, retorna p7Aux
	if(!isLoaded) {
	    return p7Aux;
	}
	
	STACK_OF(PKCS7_SIGNER_INFO) *new_si=NULL;
	STACK_OF(PKCS7_SIGNER_INFO) *old_si=NULL;
	STACK_OF(X509) *new_cert=NULL;
	STACK_OF(X509) *old_cert=NULL;	
	
	int iType =OBJ_obj2nid(p7->type);
	
	switch (iType)
	{
	   case NID_pkcs7_signedAndEnveloped:
		/* XXXXXXXXXXXXXXXX */
		new_si=p7Aux->d.signed_and_enveloped->signer_info;
		old_si=p7->d.signed_and_enveloped->signer_info;
		new_cert=p7Aux->d.signed_and_enveloped->cert;
		old_cert=p7->d.signed_and_enveloped->cert;		
		break;
	   case NID_pkcs7_signed:		
		new_si=p7Aux->d.sign->signer_info;
		old_si=p7->d.sign->signer_info;
		new_cert=p7Aux->d.sign->cert;
		old_cert=p7->d.sign->cert;
		break;
	}
	
	
	//Caso seja um pkcs7 j existente, tranfere a coleo de signerinfo do novo para o original.	
	//Signerinfo auxiliar. Usado para guardar o objeto temporrio na transferencia do pkcs7 novo para o original.
	PKCS7_SIGNER_INFO *siAux;
	
	//Percorre a coleo de signerinfo do pkcs7 novo copiando-a para o original.
	for (i=0; i<sk_PKCS7_SIGNER_INFO_num(new_si); i++)
	{
	   siAux=sk_PKCS7_SIGNER_INFO_value(new_si, i);
	   sk_PKCS7_SIGNER_INFO_push(old_si, siAux);
	}
	
	//Caso seja um pkcs7 j existente, tranfere a coleo de certificados do novo para o original.	
	//Certificado auxiliar. Usado para guardar o objeto temporrio na transferencia do pkcs7 novo para o original.
	X509 *cert;
	
	//Percorre a coleo de certificados do pkcs7 novo copiando-a para o original.
	for (i=0; i<sk_X509_num(new_cert); i++)
	{
	   cert=sk_X509_value(new_cert, i);
	   sk_X509_push(old_cert, cert);
	}		
	
	return p7;		
    }
    
    
    
  
    int pkcs7::signAndEncrypt(const char *filein, Certificate &cert, 
			      Key &privateKey, std::vector<Certificate> &certChain,  
			      std::vector<Certificate> &recipients)
    {
	FILE *fin; 
	unsigned char buf[65535];
	X509 *recipientCert = NULL;
	X509 *signerCert = cert.getX509Certificate();
	EVP_PKEY *pkey = privateKey.getPrivateKey();
	PKCS7_SIGNER_INFO *signerInfo=NULL;
    
	if((fin=fopen(filein,"rb"))!=NULL) {
	    
	    if(!digest)
		setDigest( D_SHA1 );
	    
	    if(p7) {
		PKCS7_free(p7);
		p7=NULL;
	    }
	  
	    p7 = PKCS7_new();
	  
	    if(p7) {
		 
		PKCS7_set_type(p7,NID_pkcs7_signedAndEnveloped);
		PKCS7_content_new(p7,NID_pkcs7_data);
	      
		if(!PKCS7_set_cipher(p7,cipher))
		    return PKCS7_SET_CIPHER_ERROR;
		
		std::vector<Certificate>::iterator it = recipients.begin();
		while(it != recipients.end()){
		    recipientCert = it->getX509Certificate();
		    if(!PKCS7_add_recipient(p7,recipientCert))
			return  PKCS7_ADD_RECIPIENT_ERROR;
		    it++;
		}
 		if((signerInfo = PKCS7_add_signature(p7, signerCert, pkey, digest))==NULL)
 		    return PKCS7_ADD_SIGN_ERROR ;
		
		// 		PKCS7_add_signed_attribute(signerInfo, NID_pkcs9_contentType, V_ASN1_OBJECT, OBJ_nid2obj(NID_pkcs7_data));
		// 		PKCS7_add_certificate(p7,signerCert);
		// 		PKCS7_set_detached(p7, 0);
	
 		if ((p7bio=PKCS7_dataInit(p7,NULL)) == NULL)
 		    return PKCS7_DATAINIT_ERROR;
		
 		while(true) {
		    int j = fread(buf, sizeof(char), 65535, fin);
		    if (j <= 0) break;
		    BIO_write(p7bio, buf, j);
		}
 		BIO_flush(p7bio);
 		if(!PKCS7_dataFinal(p7,p7bio))
 		    return PKCS7_DATAFINAL_ERROR;
 		BIO_free(p7bio);
		return SUCCESS;
	    }
	  
	    return PKCS7_CREATE_ERROR;
	}
	return PKCS7_FILE_CREATE_ERROR;
    }
  


    bool pkcs7::extract(pkcs8 &k, Certificate &recipient)
    {
	X509* cert = NULL;
	EVP_PKEY *pkey = NULL;

	if(&recipient != NULL)
	    cert = recipient.getX509Certificate();
	if(&k != NULL)
	    pkey = k.getKey();
	bool ret = false;
	
	if(isSignedAndEncrypted()) {
	    if( (p7bio = PKCS7_dataDecode(p7, pkey,NULL, cert)) ==NULL)
		ret = false;
	    else
		ret = true;
	} else {
	    if((p7bio = PKCS7_dataInit(p7, NULL)) == NULL)
		ret = false;
	    else 
		ret = true;
	}
	
	X509_free(cert);
	EVP_PKEY_free(pkey);
	return ret;
    }
  
  
    bool pkcs7::writeExtracted(const char* fileout)
    {
	return writeDecrypted(fileout);
    }
    
    
    
    X509_STORE *pkcs7::makeStore(const char *CAPath)
    {
	X509_STORE *store;
 	X509_LOOKUP *lookup;
 	if(!(store = X509_STORE_new())) goto end;
 	lookup=X509_STORE_add_lookup(store,X509_LOOKUP_hash_dir());
	if (lookup == NULL) goto end;
	
	if (CAPath) {
	    if(!X509_LOOKUP_add_dir(lookup, CAPath, X509_FILETYPE_ASN1)) {
		std::cout << "--> X509_LOOKUP_add_dir" << std::endl;
		goto end;
	    }
	} else {
	    X509_LOOKUP_add_dir(lookup,NULL,X509_FILETYPE_DEFAULT);
	}

	
	ERR_clear_error();
	return store;

    end:
	std::cout << "ERREUR" << std::endl;
	X509_STORE_free(store);
	return NULL;
    }
   
	
  
    /** /!\ just check signature not Certificate against CA**/
    
    int pkcs7::verify(Certificate &signer,pkcs8 &recipientPrivateKey, 
		      Certificate &recipientCert, const char* CAdir, const char* data )
    {
    
    
	X509_STORE *store = NULL;

	
	X509 *signerCertif = signer.getX509Certificate();
	
	X509 *recipientCertif = recipientCert.getX509Certificate();
	EVP_PKEY *pkey = NULL;
	
	if(&recipientPrivateKey != NULL)
	    pkey = recipientPrivateKey.getKey();
	int ret = SUCCESS;
	
	p7bio=NULL;
	
	if((!isSigned()) && (!isSignedAndEncrypted())) {
	    ret = PKCS7_BAD_CONTENT_TYPE;
	    goto end;
	}
	
	if( (store = makeStore(CAdir))==NULL) {
	    ret = PKCS7_STORE_ERROR;
	    goto end;
	}


	if(data) {
	    if(!(p7bio = BIO_new_file(data, "rb"))){
		ret =  PKCS7_BIO_NEW_ERROR;;
		goto end;
	    }
	}
	

	if(!data){
	    if(!isSignedAndEncrypted()) {
		p7bio = PKCS7_dataInit(p7,NULL);
	    } else {
		p7bio = PKCS7_dataDecode(p7,pkey,NULL, recipientCertif);
	    }
	}


	if(PKCS7_verify(p7, NULL, store , p7bio, NULL, PKCS7_NOVERIFY) != 1){
	    ret = PKCS7_SIGNATURE_VERIF_FAILED;
	}
	
    end:
	BIO_free(p7bio);
	X509_free(signerCertif);
	//X509_free(extractedSignerCertif);
	X509_free(recipientCertif);
	X509_STORE_free(store);

	return ret;
    }




/* in progress *****

*/
    std::vector<Certificate> *pkcs7::getChainFromSigned(std::vector<Certificate> *CAList, Certificate signerCert)
    {
	//get certificate from the pkcs7 
	std::vector<Certificate> vect = getCertificates();
	std::vector<Certificate>::iterator CAit = CAList->begin();
	
	hash_set<Certificate, Certificate::Certificate_hash_str, Certificate::eqstr> CAhash;
	
	while(CAit != CAList->end()) {
	    CAhash.insert(*CAit);
	    CAit++;
	}	

	//signer is the last element of the vector
	//Certificate signerCert = *it;	
	/*while(it != vect.end()) {
	    CAhash.insert(*it);
	    signerCert = *it;
	    it++;
	}*/
		
	std::cout << "SIGNER :" << signerCert.getHash() << std::endl;
	
	std::vector<Certificate> *CAmerged = new std::vector<Certificate>;
	hash_set<Certificate, Certificate::Certificate_hash_str, Certificate::eqstr>::iterator hashIt;
	
	for(hashIt = CAhash.begin() ; hashIt != CAhash.end() ; hashIt++) {
	    CAmerged->push_back(*hashIt);
	}
	
	std::vector<Certificate> *final = signerCert.getPathToRoot(CAmerged);
	CAmerged->clear();
	CAhash.clear();
	
	return final;
    }


    bool pkcs7::verifyCert(hash_map<Certificate, std::vector<Crl>, 
			   Certificate::Certificate_hash_str, Certificate::eqstr> CACRL,  
			   std::vector<Certificate> *CA, 
			   void (*callback)(std::string filename, std::string m, 
					    void *statusDialog, bool ok), 
			   std::string filename , void *statusDialog) 
    {
	return true;
    }

    
    bool pkcs7::isSigned()
    { 
	return PKCS7_type_is_signed(p7)==1;
    }

    bool pkcs7::isEncrypted()
    {
	return (OBJ_obj2nid((p7)->type) ==  NID_pkcs7_enveloped);
    }

    bool pkcs7::isSignedAndEncrypted()
    { 
	return PKCS7_type_is_signedAndEnveloped(p7)==1;
	//(OBJ_obj2nid((p7)->type) ==  NID_pkcs7_signedAndEnveloped);
    }
  

    bool pkcs7::isDetached()
    {
	return PKCS7_is_detached(p7) == 1;
	//	return (p7->d.sign->contents->d.data == NULL);
    }



    std::vector<Certificate> &pkcs7::getCertificates()
    {
	STACK_OF(X509) *stack=sk_X509_new_null();
	X509 *cert=NULL;
	std::vector<Certificate> *certVect = new std::vector<Certificate>;
	
	// certs from  pkcs7_signed_st (see crypto/pkcs7/pkcs7.h)
	if(isSigned()) {
	    //stack of 509
	    stack = sk_X509_dup(p7->d.sign->cert);
	}
    
	//cert from pkcs7_signedandenveloped_st
	else if(isSignedAndEncrypted()) {
	    stack = sk_X509_dup(p7->d.sign->cert);
	}
    
	while((cert=sk_X509_pop(stack))!=NULL) {
	    Certificate *certif = new Certificate();
	    certif->setX509Certificate(cert);
	    //certif=cert;
	    certVect->push_back(*certif);    
	    // X509_free(cert); //pb with X509_dup ... it seems to not be a real dup
	} 
	sk_X509_free(stack);
	return *certVect;
    }
  
  
    std::vector<Certificate> &pkcs7::getRecipientsCertificates() 
    {
	STACK_OF(PKCS7_RECIP_INFO) *recipientStack = NULL;
	std::vector<Certificate> *certVect = new std::vector<Certificate>;
	X509 *cert=NULL;
	
	if(isEncrypted())
	    recipientStack = sk_PKCS7_RECIP_INFO_dup(p7->d.enveloped->recipientinfo);
    
	else if(isSignedAndEncrypted())
	    recipientStack = sk_PKCS7_RECIP_INFO_dup(p7->d.signed_and_enveloped->recipientinfo);
    
	while((cert=sk_PKCS7_RECIP_INFO_pop(recipientStack)->cert) != NULL) {
	    Certificate *certif = new Certificate();
	    certif->setX509Certificate(cert);
	    std::cout << certif->getHash() << std::endl;
	    certVect->push_back(*certif);    
	    X509_free(cert);
	}

	sk_PKCS7_RECIP_INFO_free(recipientStack);
	return *certVect;
    }



    std::vector<Crl> &pkcs7::getCRL() 
    {
	STACK_OF(X509_CRL) *crlStack=NULL;
	std::vector<Crl> *crlVect = new std::vector<Crl>;
	X509_CRL *c = NULL;
	Crl *crl = NULL; 
    
	if(isSigned())
	    crlStack = sk_X509_CRL_dup(p7->d.sign->crl);
    
	else if(isSignedAndEncrypted())
	    crlStack = sk_X509_CRL_dup(p7->d.signed_and_enveloped->crl);
    
	while( (c = sk_X509_CRL_pop(crlStack)) !=NULL ){
	    crl = new Crl(c);
	    crlVect->push_back(*crl);
	}
    
	X509_CRL_free(c);
	sk_X509_CRL_free(crlStack);
	
	return *crlVect;
    }

 

    std::vector<PKCS7_RECIP_INFO> &pkcs7::getRecipientInfos()
    {
	std::vector<PKCS7_RECIP_INFO> *infos = new std::vector<PKCS7_RECIP_INFO>(0);

	if( p7 != NULL ) {
	    int i;
	    i=OBJ_obj2nid(p7->type);
	    STACK_OF(PKCS7_RECIP_INFO) *sk;
	    PKCS7_RECIP_INFO *recip = NULL;
	
	    switch (i) {
	    case NID_pkcs7_signedAndEnveloped:
		sk = sk_PKCS7_RECIP_INFO_dup(p7->d.signed_and_enveloped->recipientinfo);
		break;
	    case NID_pkcs7_enveloped:
		sk = sk_PKCS7_RECIP_INFO_dup(p7->d.enveloped->recipientinfo);
		break;
	    default:
		return *infos;
	    }
	
	    while( (recip = sk_PKCS7_RECIP_INFO_pop(sk)) !=NULL ) {
		infos->push_back(*recip);
	    }
	}
	return *infos;
     }

     std::vector<PKCS7_SIGNER_INFO> &pkcs7::getSignerInfos()
    {
	std::vector<PKCS7_SIGNER_INFO> *infos = new std::vector<PKCS7_SIGNER_INFO>(0);

	if( p7 != NULL ) {
	    int i;
	    i=OBJ_obj2nid(p7->type);
	    STACK_OF(PKCS7_SIGNER_INFO) *sk;
	    PKCS7_SIGNER_INFO *signer = NULL;
	
	    switch (i) {
	    case NID_pkcs7_signedAndEnveloped:
		sk = sk_PKCS7_SIGNER_INFO_dup(p7->d.signed_and_enveloped->signer_info);
		break;
	    case NID_pkcs7_signed:
		sk = sk_PKCS7_SIGNER_INFO_dup(p7->d.sign->signer_info);
		break;
	    default:
		return *infos;
	    }
	
	    while( (signer = sk_PKCS7_SIGNER_INFO_pop(sk)) !=NULL ) {
		infos->push_back(*signer);
	    }
	}
	return *infos;
     }


    Certificate& pkcs7::getSignerCertificate(PKCS7_SIGNER_INFO *signer)
    {
    
      std::vector<Certificate> certList = getCertificates();      
    
      for( std::vector<Certificate>::iterator it = certList.begin() ; it != certList.end() ; it++ ) {
      
      	ASN1_INTEGER *n=signer->issuer_and_serial->serial;
	BIGNUM *bn=ASN1_INTEGER_to_BN(n,NULL);
	std::string sn(BN_bn2hex(bn));
	
	if((X509_NAME_cmp(signer->issuer_and_serial->issuer , it->getIssuerName().getX509Name())==0) && (sn == it->getSerialNumber())) {
	    
	    Certificate *certif = new Certificate();
	    certif->setX509Certificate(it->getX509Certificate());
	
	    return *certif;
	}	
      }
      
      Certificate *empty = new Certificate();
      
      return *empty;	
    }

}
