//
// C++ Implementation: %{MODULE}
//
// Description:
//
//
// Author: %{AUTHOR} <%{EMAIL}>, (C) %{YEAR}
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "tokenkeystore.h"
#include <string>
#include <vector>
#include <iostream>
#include <cctype>

#include "ConfigDatabase.hh"
#include "Utils.hh"
#include "env.hh"
#include "SoftKeyStore.hh"
#include "tokenkey.h"
#include <unistd.h> //for unlink

TokenKeyStore::TokenKeyStore()
    : KeyStore()
{}

TokenKeyStore::TokenKeyStore( User *u )
{

  user = u;

  ConfigDatabase* ds = new ConfigDatabase( user->getConfigFile() );

  std::string params[] = {"", ""};
  if( ! ds->read( params ) )
  {
    std::cerr << "Cannot load configuration file '"
    <<  user->getConfigFile() << "'." <<std::endl;
    return;
  }
  OpenSSL_add_all_algorithms();

  Entry* entry;
  entry = ds->getEntry( user->getLogin() );

  if( entry )
  {
    std::vector<std::string> p12Hash = entry->getAttributeValues("PKCS12");
    std::string p12Dir = user->getP12Dir();
    std::vector<std::string>::iterator strIt;

    std::vector<std::string> certHash = entry->getAttributeValues( "X509_Certificate" );
    std::string certDir = user->getCertificatesDir();
    std::vector<std::string>::iterator certIt = certHash.begin();

    std::cout << "OpenSSL KeyStore loaded from '"
    <<  user->getConfigFile() << "'." << std::endl;
    std::cout << p12Hash.size() << " private key(s) found." << std::endl;

    for( strIt = p12Hash.begin() ; strIt != p12Hash.end() ; strIt++ , certIt++)
    {
      std::string p12Hash = *strIt;
      unsigned int posExtn = p12Hash.find(".", 0);
      std::string id = p12Hash.substr(posExtn+1, (p12Hash.size() - posExtn - 1));
      if (id == "p12")
      {
        continue;
      }
      Certificate *cert = new Certificate();
      if (cert->load(appendDir(user->getCertificatesDir() , certIt->c_str() ).c_str() ) == SUCCESS )
      {
        std::pair< std::string , Certificate > newPair (id, *cert) ;
        keysAndCerts.push_back( newPair );
      }
    }
    delete entry;
  }
  delete ds;
}

TokenKeyStore::~TokenKeyStore()
{}

/** Get the list of Certificates
 * @return a vector of Certificate
 */
std::vector<Certificate> TokenKeyStore::listCertificates()
{
  std::vector<Certificate> certs;
  for (int i=0; i<keysAndCerts.size(); i++)
  {
    certs.push_back(keysAndCerts[i].second);
  }
  return certs;
}

/** Get the Key corresponding to the given certificate.
 * Must return one of GETKEY_SUCCESS, GETKEY_CANCELED, GETKEY_INVALID, GETKEY_BAD_PASSWD
 */
int TokenKeyStore::getKey(Certificate c, Key **k)
{
    std::vector< std::pair< std::string, Certificate > >::iterator it;
    for( it = keysAndCerts.begin() ; it != keysAndCerts.end() ; it++ )
    {
      if( strcmp( it->second.getHash().c_str() , c.getHash().c_str() ) == 0 ) {
           char *passwd=NULL;
          if(pm->getPassword("", it->second.getHash(), &passwd, 0, &c))
          {
	  	std::string keyId = it->first;
		std::cout << "KeyID: " << keyId << std::endl;
	  	TokenKey *key = new TokenKey(keyId, std::string(passwd));
		if (key->getPrivateKey() == NULL) {
			delete key;
			key = NULL;
			return GETKEY_INVALID;
		}
		*k = key;		
		return GETKEY_SUCCESS;
	  }
	  else {
	  	return GETKEY_CANCELED;
	  }
      }
    }
    return -1;
}

void TokenKeyStore::setPasswordManager(PasswordManager *m)
{
	if (m != NULL) {
		pm = m;
	}
}

int TokenKeyStore::getType()
{
  return PKCS11_KEYSTORE;
}

std::vector< std::string > TokenKeyStore::listKeys(std::string report)
{
  std::vector< std::string > keys;
  std::string::size_type pos = 0;
  std::string::size_type old_pos = pos;
  std::cout << "report.size() = " << report.size() << std::endl;
  std::string keyName;
  std::string keyId;
  while ((pos < report.size()) && ((pos = report.find("\n",pos)) != std::string::npos))
  {
    std::cout << old_pos << "," << pos << ">";
    std::string tbParsed = report.substr(old_pos, (pos - old_pos));
    old_pos = ++pos;
    unsigned int internalPos = (!isspace(tbParsed[0]))? 0 : tbParsed.find_first_not_of(" \t");
    std::cout << tbParsed << std::endl;
    if (tbParsed.find("Private RSA Key", internalPos, true) == internalPos)
    {
      std::string::size_type initialPos = tbParsed.find("[", 0);
      if (initialPos != std::string::npos)
      {
        std::string::size_type finalPos = tbParsed.find("]", initialPos);
        if (finalPos != std::string::npos && (finalPos-initialPos > 0))
        {
          initialPos++;
          keyName = tbParsed.substr(initialPos, finalPos-initialPos);
	  if (keyName.size() == 0) {
	  keyName = "Sem Label";
	  }
          keyId = "";
        }
      }
    }
    if (tbParsed.find("ID", internalPos, true) == internalPos && (keyName.size() > 0))
    {
      std::string::size_type initialPos = tbParsed.find(":", 0);
      if ((initialPos != std::string::npos) && (initialPos < (tbParsed.size() - 1)))
      {
        initialPos++;
        std::string auxData = tbParsed.substr(initialPos, tbParsed.size() - initialPos);
        internalPos = (!isspace(auxData[0]))? 0 : auxData.find_first_not_of(" \t");
        keyId = auxData.substr(internalPos, auxData.size() - internalPos);
      }
      keys.push_back(keyId);
      keyName = keyId = "";
    }
  }
  std::cout << "Total de chaves encontradas: " << keys.size() << std::endl;
  return keys;
}

Certificate* TokenKeyStore::getCert(std::string keyId)
{
  Certificate *cert = NULL;
  std::vector< std::pair< std::string, Certificate> >::iterator certsIt = tempCerts.begin();
  for (; certsIt != tempCerts.end(); certsIt++)
  {
    if (certsIt->first == keyId)
    {
      cert = &certsIt->second;
    }
  }
  return cert;
}

Certificate* TokenKeyStore::decodeCert(std::string keyId, std::string certPEM)
{
  Certificate *cert = new Certificate(certPEM.c_str(), pem_format);
  std::pair<std::string, Certificate> tempCert(keyId, *cert);
  return cert;
}

int TokenKeyStore::removeKey( const size_t index , std::string &ret, const std::string password)
{
  std::cout << __FUNCTION__ << std::endl;
  std::vector<std::string> identities = user->getInfos( "Identities" );
  std::vector<std::string>::iterator strIt;
  int posToken = -1;
  unsigned int i = 0;
  std::cout << "index: " << index << std::endl;
  for (strIt = identities.begin(); ((strIt != identities.end()) && (i<=index)); strIt++)
  {
    if ((*strIt)[0] == '_')
    {
      posToken++;
    }
    i++;
  }

  std::cout << "i: " << i << ", posToken: " << posToken << std::endl;
  if( (posToken >= 0) && ( posToken < keysAndCerts.size() ) && (i>index))
  {
    std::vector< std::pair < std::string, Certificate> >::iterator it = keysAndCerts.begin();
    i = 0;
    while( it != keysAndCerts.end() && i != index )
    {
      it++;
      i++;
    }

    Certificate *cert=NULL;

    std::string certfilename;

    cert = new Certificate( it->second );
    if( !cert)
    {
      if( cert )
        delete cert;
      return -4;
    }
    // remove object
    ret = "OK";
    certfilename = setExt(cert->getHash() , CERT_DEFAULT_EXT );
    certfilename = appendDir( user->getCertificatesDir() , certfilename );

    std::vector<std::string> certInfos;
    std::vector<std::string> p12Infos;

    certInfos = user->getInfos( "X509_Certificate" );
    p12Infos = user->getInfos( "PKCS12" );
    identities = user->getInfos( "Identities" );

    p12Infos.erase( find( p12Infos.begin(),
                          p12Infos.end(),
                          std::string(cert->getHash() + "." + it->first)) );
    user->setInfos( "PKCS12", p12Infos );

    identities.erase( find( identities.begin(),
                            identities.end(),
                            identities[index]) );
    user->setInfos( "Identities", identities );


    certInfos.erase(  find( certInfos.begin(),
                            certInfos.end(),
                            std::string(cert->getHash() + ".der")) );
    user->setInfos( "X509_Certificate", certInfos );

    if( unlink( certfilename.c_str() ) == -1 )
    {
      ret = certfilename;
      ret +="\n";
      ret +=strerror(errno);
      std::cout << ret << std::endl;
      return -1;//KEYSTORE_UNLINK_ERROR;
    }

    delete cert;
    keysAndCerts.erase( it );

    return SUCCESS;
  }
  return -3;
}


void TokenKeyStore::addPair(Certificate cert, std::string keyID) {
	std::pair< std::string , Certificate > newPair (keyID, cert) ;
	keysAndCerts.push_back( newPair );
}


std::vector< std::string > TokenKeyStore::listCerts(std::string report)
{
  std::vector< std::string > certsId;
  std::string::size_type pos = 0;
  std::string::size_type old_pos = pos;
  std::cout << "report.size() = " << report.size() << std::endl;
  std::string certId;
  while ((pos < report.size()) && ((pos = report.find("\n",pos)) != std::string::npos))
  {
    std::cout << old_pos << "," << pos << ">";
    std::string tbParsed = report.substr(old_pos, (pos - old_pos));
    old_pos = ++pos;
    unsigned int internalPos = (!isspace(tbParsed[0]))? 0 : tbParsed.find_first_not_of(" \t");
    std::cout << tbParsed << std::endl;
    if (tbParsed.find("ID", internalPos, true) == internalPos)
    {
      std::string::size_type initialPos = tbParsed.find(":", 0);
      if ((initialPos != std::string::npos) && (initialPos < (tbParsed.size() - 1)))
      {
        initialPos++;
        std::string auxData = tbParsed.substr(initialPos, tbParsed.size() - initialPos);
        internalPos = (!isspace(auxData[0]))? 0 : auxData.find_first_not_of(" \t");
        certId = auxData.substr(internalPos, auxData.size() - internalPos);
      }
      std::cout << "Cert ID: " << certId << std::endl;
      certsId.push_back(certId);
      //std::pair<std::string, Certificate> temp(keyName, (Certificate)NULL);
      //tempCerts.push_back(temp);
      certId = "";
    }
  }
  return certsId;
}
